/**
 * @file greyscale_filter.cc
 *
 * @copyright 2021 3081W, All rights reserved.
 */

 /*******************************************************************************
 * Includes
 ******************************************************************************/
#include "greyscale_filter.h"
#include <iostream>
#include <string>
#include <memory>
#include <vector>
#include "image.h"

using namespace std;

/*******************************************************************************
 * Member Functions
 ******************************************************************************/

void GreyscaleFilter::Apply(std::vector<Image*> input, std::vector<Image*> output){
  int num_images = input.size();

  for(int img_count = 0; img_count < num_images; img_count++){
    int img_height = input[img_count]->GetHeight();
    int img_width = input[img_count]->GetWidth();

    output[img_count]->SetImage(img_width, img_height, 4);

    for(int y =0; y < img_height; y++){
      for(int x = 0; x < img_width; x++){
        float luminance = input[img_count]->GetLuminance(x,y);

        unique_ptr<unsigned char[]> newPixel = unique_ptr<unsigned char[]>(new unsigned char[4]);
        unsigned char* newPixel_ptr = newPixel.get();
        newPixel_ptr[0] = static_cast<int>(luminance);
        newPixel_ptr[1] = static_cast<int>(luminance);
        newPixel_ptr[2] = static_cast<int>(luminance);
        newPixel_ptr[3] = 255;

        output[img_count]->SetPixel(x, y, newPixel_ptr);


      }
    }
  }
}
